@@ -177,11 +177,11 @@ pub struct SequenceFormat {
     pub format_string: String,      // sequence with 'N's replacing barcodes
     pub regions_string: String,     // String with each region contain a code
     pub length: usize,              // Total length of format sequence
-    pub constant_region_length: u8, // Length of only the consant nucleotides
+    pub constant_region_length: u16, // Length of only the consant nucleotides
     pub format_regex: Regex,        // The regex search used to find barcodes
     pub barcode_num: usize,         // Number of counted barcodes.  More for DEL
-    pub barcode_lengths: Vec<u8>,   // The length of each counted barcode
-    pub sample_length_option: Option<u8>, // Sample barcode length
+    pub barcode_lengths: Vec<u16>,   // The length of each counted barcode
+    pub sample_length_option: Option<u16>, // Sample barcode length
     pub random_barcode: bool,       // Whether a random barcode is included
     pub sample_barcode: bool,       // Whether a sammple barcode is included
 }
@@ -255,7 +255,7 @@ impl SequenceFormat {
                     .get(0)
                     .unwrap()
                     .as_str()
-                    .parse::<u8>()
+                    .parse::<u16>()
                     .unwrap();
 
                 // Create the capture group with the group name for the barcode and add it to the
@@ -301,7 +301,7 @@ impl SequenceFormat {
                 for _ in 0..constant_group_length {
                     sequence_format.regions_string.push('C');
                 }
-                sequence_format.constant_region_length += constant_group_length as u8;
+                sequence_format.constant_region_length += constant_group_length as u16;
             }
         }
         sequence_format.length = sequence_format.format_string.chars().count();
@@ -460,14 +460,14 @@ impl BarcodeConversions {
 #[derive(Debug, Clone, PartialEq)]
 pub struct MaxSeqErrors {
     // errors within the constant region
-    constant_region: u8,
-    constant_region_size: u8,
+    constant_region: u16,
+    constant_region_size: u16,
     // errors within the sample barcode
-    sample_barcode: u8,
-    sample_size: u8,
+    sample_barcode: u16,
+    sample_size: u16,
     // erors within the counted barcode
-    barcode: Vec<u8>,
-    barcode_sizes: Vec<u8>,
+    barcode: Vec<u16>,
+    barcode_sizes: Vec<u16>,
     min_quality: f32,
 }
 
@@ -488,12 +488,12 @@ impl MaxSeqErrors {
     /// let mut max_sequence_errors = MaxSeqErrors::new(sample_errors_option, sample_barcode_size_option, barcode_errors_option, barcode_sizes, constant_errors_option, constant_region_size, min_quality);
     /// ```
     pub fn new(
-        sample_errors_option: Option<u8>,
-        sample_barcode_size_option: Option<u8>,
-        barcode_errors_option: Option<u8>,
-        barcode_sizes: Vec<u8>,
-        constant_errors_option: Option<u8>,
-        constant_region_size: u8,
+        sample_errors_option: Option<u16>,
+        sample_barcode_size_option: Option<u16>,
+        barcode_errors_option: Option<u16>,
+        barcode_sizes: Vec<u16>,
+        constant_errors_option: Option<u16>,
+        constant_region_size: u16,
         min_quality: f32,
     ) -> Self {
         let max_sample_errors;
@@ -562,7 +562,7 @@ impl MaxSeqErrors {
     /// let mut max_sequence_errors = MaxSeqErrors::new(sample_errors_option, sample_barcode_size_option, barcode_errors_option, barcode_sizes, constant_errors_option, constant_region_size, min_quality);
     /// assert_eq!(max_sequence_errors.max_constant_errors(), 3);
     /// ```
-    pub fn max_constant_errors(&self) -> u8 {
+    pub fn max_constant_errors(&self) -> u16 {
         self.constant_region
     }
 
@@ -586,7 +586,7 @@ impl MaxSeqErrors {
     /// let mut max_sequence_errors = MaxSeqErrors::new(sample_errors_option, sample_barcode_size_option, barcode_errors_option, barcode_sizes, constant_errors_option, constant_region_size, min_quality);
     /// assert_eq!(max_sequence_errors.max_sample_errors(), 3);
     /// ```
-    pub fn max_sample_errors(&self) -> u8 {
+    pub fn max_sample_errors(&self) -> u16 {
         self.sample_barcode
     }
 
@@ -610,7 +610,7 @@ impl MaxSeqErrors {
     /// let mut max_sequence_errors = MaxSeqErrors::new(sample_errors_option, sample_barcode_size_option, barcode_errors_option, barcode_sizes, constant_errors_option, constant_region_size, min_quality);
     /// assert_eq!(max_sequence_errors.max_barcode_errors(), vec![2,2,2]);
     /// ```
-    pub fn max_barcode_errors(&self) -> &[u8] {
+    pub fn max_barcode_errors(&self) -> &[u16] {
         &self.barcode
     }
 }
@@ -658,20 +658,17 @@ impl fmt::Display for MaxSeqErrors {
     }
 }
 
-// An enum for whether or not the sequencing format contains a random barcode.  This changes how the barcodes are counted and which are kept
 #[derive(Debug)]
-pub enum FormatType {
-    RandomBarcode,
-    NoRandomBarcode,
+pub enum ResultsHashmap {
+    RandomBarcode(HashMap<String, HashMap<String, HashSet<String>>>),
+    NoRandomBarcode(HashMap<String, HashMap<String, usize>>),
 }
 
 // A struct which holds the count results, whether that is for a scheme which contains a random barcode or not
 #[derive(Debug)]
 pub struct Results {
-    pub random_hashmap: HashMap<String, HashMap<String, HashSet<String>>>, // The counts for for schemes that contain random barcodes
-    pub count_hashmap: HashMap<String, HashMap<String, u32>>, // The counts for the schemes which don't contain random barcodes.  Right now it is either on or the other.  Too much memory may be needed otherwise
-    pub format_type: FormatType, // Whether it is with a random barcode or not
-    empty_count_hash: HashMap<String, u32>, // An empty hashmap that is used a few times and therefor stored within the struct
+    pub results_hashmap: ResultsHashmap, // holds the counted results
+    empty_count_hash: HashMap<String, usize>, // An empty hashmap that is used a few times and therefor stored within the struct
     empty_random_hash: HashMap<String, HashSet<String>>,
     sample_conversion_omited: bool,
 }
@@ -683,120 +680,145 @@ impl Results {
         random_barcode: bool,
         sample_barcode: bool,
     ) -> Self {
-        // Record and keep the format type of whether or not the random barcode is included
-        let format_type;
+        let mut results_hashmap;
+        // Create an empty hashmap into the enum depending on whether or not a random barcode is
+        // included
         if random_barcode {
-            format_type = FormatType::RandomBarcode
+            results_hashmap = ResultsHashmap::RandomBarcode(HashMap::new());
         } else {
-            format_type = FormatType::NoRandomBarcode
+            results_hashmap = ResultsHashmap::NoRandomBarcode(HashMap::new());
         }
 
-        let mut random_hashmap = HashMap::new(); // create the hashmap that is used to count with random barcode scheme
-        let mut count_hashmap = HashMap::new(); // create the hashmap that is used ot count when a random barcode is not used.  One or the other stays empty depending on the scheme
-
-        // create empty hashmaps to insert and have the sample name included.  This is so sample name doesn't need to be searched each time
-        let empty_random_hash: HashMap<String, HashSet<String>> = HashMap::new();
-        let empty_count_hash: HashMap<String, u32> = HashMap::new();
-
         // If sample name conversion was included, add all sample names to the hashmaps used to count
         let mut sample_conversion_omited = false;
+        // create empty hashmaps to insert and have the sample name included.  This is so sample name doesn't need to be searched each time
+        let empty_random_hash: HashMap<String, HashSet<String>> = HashMap::new();
+        let empty_count_hash: HashMap<String, usize> = HashMap::new();
+        // If there is a sample barcode file included, add these as keys in the relevant count hashmap
         if !samples_barcode_hash.is_empty() {
             for sample in samples_barcode_hash.keys() {
                 let sample_barcode = sample.to_string();
-                random_hashmap.insert(sample_barcode.clone(), empty_random_hash.clone());
-                count_hashmap.insert(sample_barcode, empty_count_hash.clone());
+                match results_hashmap {
+                    ResultsHashmap::RandomBarcode(ref mut random_hashmap) => {
+                        random_hashmap.insert(sample_barcode.clone(), empty_random_hash.clone());
+                    }
+                    ResultsHashmap::NoRandomBarcode(ref mut count_hashmap) => {
+                        count_hashmap.insert(sample_barcode, empty_count_hash.clone());
+                    }
+                }
             }
         } else if !sample_barcode {
-            // If sample names are not included, insert unknown name into the hashmaps
-            random_hashmap.insert("barcode".to_string(), empty_random_hash.clone());
-            count_hashmap.insert("barcode".to_string(), empty_count_hash.clone());
+            // If there is not a sample barcode within the format, add 'barcode' as key
+            match results_hashmap {
+                ResultsHashmap::RandomBarcode(ref mut random_hashmap) => {
+                    random_hashmap.insert("barcode".to_string(), empty_random_hash.clone());
+                }
+                ResultsHashmap::NoRandomBarcode(ref mut count_hashmap) => {
+                    count_hashmap.insert("barcode".to_string(), empty_count_hash.clone());
+                }
+            }
         } else {
+            // If there is a sample barcode in the format but no sample barcode conversion file,
+            // set the following to true to make sample DNA barcodes into keys later on
             sample_conversion_omited = true;
         }
         // return the Results struct
         Results {
-            random_hashmap,
-            count_hashmap,
-            format_type,
+            results_hashmap,
             empty_count_hash,
             empty_random_hash,
             sample_conversion_omited,
         }
     }
 
-    /// Adds the random barcode connected to the barcode_ID that is counted and the sample.  When writing these unique barcodes are counted to get a count
-    pub fn add_random(
+    /// Adds the count to results hashmap
+    pub fn add_count(
         &mut self,
         sample_barcode: &str,
-        random_barcode: &str,
+        random_barcode: Option<&String>,
         barcode_string: String,
     ) -> bool {
-        if self.sample_conversion_omited && !self.random_hashmap.contains_key(sample_barcode) {
-            self.random_hashmap
-                .insert(sample_barcode.to_string(), self.empty_random_hash.clone());
-        };
-
-        // Get the hashmap for the sample
-        let barcodes_hashmap_option;
-        if sample_barcode.is_empty() {
-            barcodes_hashmap_option = self.random_hashmap.get_mut("barcode");
-        } else {
-            barcodes_hashmap_option = self.random_hashmap.get_mut(sample_barcode);
-        }
-        if let Some(barcodes_hashmap) = barcodes_hashmap_option {
-            // If the barcodes_hashmap is not empty
-            // but doesn't contain the barcode
-            if !barcodes_hashmap.contains_key(&barcode_string) {
-                // insert the hashmap<barcode_id, Set<random_barcodes>>
-                let mut intermediate_set = HashSet::new();
-                intermediate_set.insert(random_barcode.to_string());
-                barcodes_hashmap.insert(barcode_string, intermediate_set);
-            } else {
-                // if the hashmap<sample_id, hashmap<barcode_id, Set<>> exists, check to see if the random barcode already was inserted
-                let random_set = barcodes_hashmap.get_mut(&barcode_string).unwrap();
-                return random_set.insert(random_barcode.to_string());
+        // If conversion file does not exist, add the barcode as a key value
+        if self.sample_conversion_omited {
+            match self.results_hashmap {
+                ResultsHashmap::NoRandomBarcode(ref mut count_hashmap) => {
+                    if !count_hashmap.contains_key(sample_barcode) {
+                        count_hashmap
+                            .insert(sample_barcode.to_string(), self.empty_count_hash.clone());
+                    };
+                }
+                ResultsHashmap::RandomBarcode(ref mut random_hashmap) => {
+                    if !random_hashmap.contains_key(sample_barcode) {
+                        random_hashmap
+                            .insert(sample_barcode.to_string(), self.empty_random_hash.clone());
+                    };
+                }
             }
-        } else {
-            // create the Set<RandomBarcode>
-            let mut intermediate_set = HashSet::new();
-            intermediate_set.insert(random_barcode.to_string());
-            let mut intermediate_hash = HashMap::new();
-            // create the HashMap<barcode_id, Set<RandomBarcodes>>
-            intermediate_hash.insert(barcode_string.to_string(), intermediate_set);
-            // insert this into the random_hashmap connected to the sample_ID
-            self.random_hashmap
-                .insert(sample_barcode.to_string(), intermediate_hash);
-        }
-        true
-    }
-
-    /// Adds to the count for the barcode_id connected to the sample. This is used when a random barcode is not included in the scheme
-    pub fn add_count(&mut self, sample_barcode: &str, barcode_string: String) {
-        if self.sample_conversion_omited && !self.count_hashmap.contains_key(sample_barcode) {
-            self.count_hashmap
-                .insert(sample_barcode.to_string(), self.empty_count_hash.clone());
         };
-        // Insert 0 if the barcodes are not within the sample_name -> barcodes
-        // Then add one regardless
-        *self
-            .count_hashmap
-            .get_mut(sample_barcode)
-            .unwrap_or(&mut self.empty_count_hash.clone())
-            .entry(barcode_string)
-            .or_insert(0) += 1;
+
+        match self.results_hashmap {
+            // If random barcode is not included, add the count to this hashmap
+            ResultsHashmap::NoRandomBarcode(ref mut count_hashmap) => {
+                *count_hashmap
+                    .get_mut(sample_barcode)
+                    .unwrap_or(&mut self.empty_count_hash.clone())
+                    .entry(barcode_string)
+                    .or_insert(0) += 1;
+            }
+            // If a random barcode is included, add the random barcode and later use the number of
+            // random barcodes as the count
+            ResultsHashmap::RandomBarcode(ref mut random_hashmap) => {
+                // Get the hashmap for the sample
+                let barcodes_hashmap_option;
+                if sample_barcode.is_empty() {
+                    barcodes_hashmap_option = random_hashmap.get_mut("barcode");
+                } else {
+                    barcodes_hashmap_option = random_hashmap.get_mut(sample_barcode);
+                }
+                if let Some(barcodes_hashmap) = barcodes_hashmap_option {
+                    // If the barcodes_hashmap is not empty
+                    // but doesn't contain the barcode
+                    if !barcodes_hashmap.contains_key(&barcode_string) {
+                        // insert the hashmap<barcode_id, Set<random_barcodes>>
+                        let mut intermediate_set = HashSet::new();
+                        intermediate_set
+                            .insert(random_barcode.unwrap_or(&"".to_string()).to_string());
+                        barcodes_hashmap.insert(barcode_string, intermediate_set);
+                    } else {
+                        // if the hashmap<sample_id, hashmap<barcode_id, Set<>> exists, check to see if the random barcode already was inserted
+                        let random_set = barcodes_hashmap.get_mut(&barcode_string).unwrap();
+                        return random_set
+                            .insert(random_barcode.unwrap_or(&"".to_string()).to_string());
+                    }
+                } else {
+                    // create the Set<RandomBarcode>
+                    let mut intermediate_set = HashSet::new();
+                    intermediate_set.insert(random_barcode.unwrap_or(&"".to_string()).to_string());
+                    let mut intermediate_hash = HashMap::new();
+                    // create the HashMap<barcode_id, Set<RandomBarcodes>>
+                    intermediate_hash.insert(barcode_string.to_string(), intermediate_set);
+                    // insert this into the random_hashmap connected to the sample_ID
+                    random_hashmap.insert(sample_barcode.to_string(), intermediate_hash);
+                }
+            }
+        }
+
+        // Return that a count was added.  An earlier return value is used for when a random
+        // barcode is already within the results
+        true
     }
 }
 
 /// A struct which holds hte enriched single and double counted barcodes.  Useful for DEL.  This struct is used during output.
 pub struct ResultsEnrichment {
-    pub single_hashmap: HashMap<String, HashMap<String, u32>>, // enrichment of single barcodes hash used at output
-    pub double_hashmap: HashMap<String, HashMap<String, u32>>, // enrichment of double barcodes hash used at output
-    empty_count_hash: HashMap<String, u32>,
+    pub single_hashmap: HashMap<String, HashMap<String, usize>>, // enrichment of single barcodes hash used at output
+    pub double_hashmap: HashMap<String, HashMap<String, usize>>, // enrichment of double barcodes hash used at output
+    empty_count_hash: HashMap<String, usize>,
 }
 
 impl ResultsEnrichment {
     pub fn new() -> Self {
-        let empty_count_hash: HashMap<String, u32> = HashMap::new();
+        let empty_count_hash: HashMap<String, usize> = HashMap::new();
         ResultsEnrichment {
             single_hashmap: HashMap::new(),
             double_hashmap: HashMap::new(),
@@ -816,7 +838,7 @@ impl ResultsEnrichment {
     }
 
     /// Adds the count the the single barcode enrichment hashmap
-    pub fn add_single(&mut self, sample_id: &str, barcode_string: &str, count: u32) {
+    pub fn add_single(&mut self, sample_id: &str, barcode_string: &str, count: usize) {
         // get the number of barcodes to know homu much to iterate
         let barcode_num = barcode_string.split(',').count();
         // For each single barcode in the comma separate barcodes, create a new string with just one barcode and empty other columns
@@ -845,7 +867,7 @@ impl ResultsEnrichment {
     }
 
     /// Adds the count to the double barcode enrichment hashmap
-    pub fn add_double(&mut self, sample_id: &str, barcode_string: &str, count: u32) {
+    pub fn add_double(&mut self, sample_id: &str, barcode_string: &str, count: usize) {
         // get the number of barcodes to know homu much to iterate
         let barcode_num = barcode_string.split(',').count();
         // split the barcodes into a vec from their comma separated form
